from typing import List

import pytest

from gcs_migration.models import (
    ACL,
    CredentialYamlDict,
    PosixCredential,
    PosixGuestCollection,
    S3Credential,
    S3CredentialYamlDict,
    S3GuestCollection,
)
from gcs_migration.typing.transfer_documents import (
    AccessListDocumentDict,
    AnonymousEffectivePermissionsDict,
    EndpointDocumentDict,
    RoleListDocumentDict,
)


@pytest.fixture
def guest_collection(
    shared_endpoint_document: EndpointDocumentDict,
    access_list_document: AccessListDocumentDict,
    share_role_list_document: RoleListDocumentDict,
    anonymous_effective_permissions_document: AnonymousEffectivePermissionsDict,
    share_state_file: str,
    credential: CredentialYamlDict,
) -> PosixGuestCollection:
    guest_collection = PosixGuestCollection()
    guest_collection.transfer_endpoint = shared_endpoint_document
    guest_collection.access_list = access_list_document
    guest_collection.role_list = share_role_list_document
    guest_collection.anonymous_effective_permissions = (
        anonymous_effective_permissions_document
    )
    guest_collection.share_state_file = share_state_file

    cred = PosixCredential(credential)
    guest_collection.credential = cred
    guest_collection.credential_id = cred.id

    return guest_collection


@pytest.fixture
def s3_guest_collection(
    shared_endpoint_document: EndpointDocumentDict,
    access_list_document: AccessListDocumentDict,
    share_role_list_document: RoleListDocumentDict,
    anonymous_effective_permissions_document: AnonymousEffectivePermissionsDict,
    share_state_file: str,
    s3_credential: S3CredentialYamlDict,
) -> S3GuestCollection:
    guest_collection = S3GuestCollection()
    guest_collection.transfer_endpoint = shared_endpoint_document
    guest_collection.access_list = access_list_document
    guest_collection.role_list = share_role_list_document
    guest_collection.anonymous_effective_permissions = (
        anonymous_effective_permissions_document
    )
    guest_collection.share_state_file = share_state_file

    s3_cred = S3Credential(s3_credential)
    guest_collection.credential = s3_cred
    guest_collection.credential_id = s3_cred.id

    return guest_collection


@pytest.fixture
def guest_collection_unresolved_credential(
    shared_endpoint_document: EndpointDocumentDict,
    access_list_document: AccessListDocumentDict,
    anonymous_effective_permissions_document: AnonymousEffectivePermissionsDict,
    share_state_file: str,
    credential_unresolved: CredentialYamlDict,
) -> PosixGuestCollection:
    guest_collection = PosixGuestCollection()
    guest_collection.transfer_endpoint = shared_endpoint_document
    guest_collection.access_list = access_list_document
    guest_collection.anonymous_effective_permissions = (
        anonymous_effective_permissions_document
    )
    guest_collection.share_state_file = share_state_file

    cred = PosixCredential(credential_unresolved)
    guest_collection.credential = cred
    guest_collection.credential_id = cred.id

    return guest_collection


@pytest.fixture
def new_acls() -> List[ACL]:
    return [
        ACL(
            data={
                "id": id,
                "principal": principal,
                "principal_type": "identity",
                "path": "/some-new-acl-path",
                "permissions": "rw",
                "create_time": f"2021-04-06T19:{id % 60}:08+00:00",
            }
        )
        for (id, principal) in [
            (42, "6024d5b9-d843-47d7-8c98-4c3673cb43e3"),
            (314, "6c200665-e635-40bf-8726-53602ed26d56"),
        ]
    ]
