from typing import List, Optional

import pytest
from pyfakefs.fake_filesystem import FakeFilesystem

from gcs_migration.parsers import S3MapEntry, S3MapfileParser


def test_s3_mapfile_parser(
    fs: FakeFilesystem,
    s3_map_filename: str,
    s3_map_files: List[str],
    s3_map_entries: List[S3MapEntry],
    user_without_mapfile: str,
    s3_process_user: Optional[str],
) -> None:
    map_parser = S3MapfileParser(s3_map_filename, process_user=s3_process_user)

    for entry in s3_map_entries:
        assert entry.username in map_parser
        assert map_parser[entry.username] == entry


def test_s3_mapfile_parser_invalid_user(
    fs: FakeFilesystem,
    s3_map_filename: str,
    s3_map_files: List[str],
    s3_map_entries: List[S3MapEntry],
    user_without_mapfile: str,
) -> None:
    map_parser = S3MapfileParser(s3_map_filename)

    assert "nonexistant_user" not in s3_map_entries
    with pytest.raises(KeyError):
        map_parser["nonexistant_user"]


def test_s3_mapfile_parser_no_user_mapfile(
    fs: FakeFilesystem,
    s3_map_filename: str,
    s3_map_files: List[str],
    s3_map_entries: List[S3MapEntry],
    user_without_mapfile: str,
) -> None:
    map_parser = S3MapfileParser(s3_map_filename)

    assert user_without_mapfile not in s3_map_entries
    with pytest.raises(KeyError):
        map_parser[user_without_mapfile]


def test_s3_mapfile_parser_not_found(
    fs: FakeFilesystem,
) -> None:
    with pytest.raises(IOError):
        S3MapfileParser("/not-existing-file")


def test_s3_mapfile_parser_default_path(
    fs: FakeFilesystem,
) -> None:
    parser = S3MapfileParser()
    assert parser.path is not None
